
(*Title graphicx*)


PseudoAngle[v_] := Module[{ax, ay, t}, ax = Abs[v[[1]]]; ay = Abs[v[[2]]];
      If[ax + ay == 0, t = 0, t = v[[2]]/(ax + ay)];
      If[v[[1]] < 0, t = 2 - t, If[v[[2]] < 0, t += 4, 0]];
      Return[t*90];];



AngleBetween[v1_, v2_] := 
    Module[{angle = PseudoAngle[v2] - PseudoAngle[v1]}, 
      If[angle >= 0., Return[angle], Return[360 + angle]]];



SortVertices[vlst_]:=
    Module[{mean=0,nlst={}},If[Length[vlst]>1,mean=1/Length[vlst] Plus@@vlst;
          nlst=Transpose[vlst];          
          nlst=Transpose[
              AppendTo[nlst,Map[PseudoAngle,Map[(#-mean)&,vlst]]]];
          nlst=Sort[nlst,OrderedQ[{#1[[3]],#2[[3]]}]&];
          nlst=Transpose[Take[Transpose[nlst],2]];
          Return[nlst],Return[vlst]];];


CleanPoly[poly_] := 
    Module[{temp = {}}, 
      If[Length[poly] > 2, 
          Do[If[Chop[poly[[i]] - poly[[Mod[i, Length[poly]] + 1]]] != {0., 
                  0.}, AppendTo[temp, poly[[i]]], 0], {i, Length[poly]}];
          Return[temp];, Return[{}]];];



GraphPoly[vlst_] := 
    Graphics[Table[{Thickness[0.0001], 
          Line[{vlst[[i]], vlst[[Mod[i, Length[vlst]] + 1]]}]}, {i, 
          Length[vlst]}]];

DrawPolys[polylist_, as_] := 
    Show[Map[GraphPoly, polylist], AspectRatio -> as];

DrawFilledPolys[polys_] := 
    Module[{xmin, xmax, ymin, ymax, as}, 
      xmin = Min[Transpose[Flatten[polys, 1]][[1]]];
      xmax = Max[Transpose[Flatten[polys, 1]][[1]]];
      ymin = Min[Transpose[Flatten[polys, 1]][[2]]];
      ymax = Max[Transpose[Flatten[polys, 1]][[2]]];
      as = (ymax - ymin)/(xmax - xmin);
      Show[
        Graphics[
          Table[{Hue[i/Length[polys]], Polygon[polys[[i]]]}, {i, 
              Length[polys]}]], PlotRange -> {{xmin, xmax}, {ymin, ymax}}, 
        AspectRatio -> as];];

DrawFilledPolysQuiet[polys_] := 
    Module[{xmin, xmax, ymin, ymax, as}, 
      xmin = Min[Transpose[Flatten[polys, 1]][[1]]];
      xmax = Max[Transpose[Flatten[polys, 1]][[1]]];
      ymin = Min[Transpose[Flatten[polys, 1]][[2]]];
      ymax = Max[Transpose[Flatten[polys, 1]][[2]]];
      as = (ymax - ymin)/(xmax - xmin);
      Return[
        Show[Graphics[
            Table[{Hue[i/Length[polys]], Polygon[polys[[i]]]}, {i, 
                Length[polys]}]], PlotRange -> {{xmin, xmax}, {ymin, ymax}}, 
          AspectRatio -> as, DisplayFunction -> Identity]];];

vertical[seg_] := If[Chop[seg[[1]][[1]] - seg[[2]][[1]]] == 0, True, False];


IsInSeg[seg_, x_] := 
    Module[{a, b, test = True, absc = Transpose[seg][[1]], 
        ord = Transpose[seg][[2]]}, 
      If[Chop[x[[1]] - Max[absc]] > 0. || Chop[x[[1]] - Min[absc]] < 0., 
        test = False, 0];
      If[Chop[x[[2]] - Max[ord]] > 0. || Chop[x[[2]] - Min[ord]] < 0., 
        test = False, 0];
      If[! vertical[seg], 
        a = (seg[[2]][[2]] - seg[[1]][[2]])/(seg[[2]][[1]] - seg[[1]][[1]]);
        b = seg[[1]][[2]] - a*seg[[1]][[1]];
        If[Chop[x[[2]] - a*x[[1]] - b] != 0, test = False, 0];, 0];
      Return[test];];



IsInPoly[vlst_, x_] := 
    Module[{tlst = Map[(# - x) &, vlst], test = True}, 
      Do[If[Chop[
              AngleBetween[tlst[[i]], tlst[[Mod[i, Length[tlst]] + 1]]] - 
                180.] > 0., test = False, 0], {i, Length[vlst]}];
      Do[If[Chop[vlst[[i]] - x] == {0, 0}, test = True, 0];, {i, 
          Length[vlst]}];
      Return[test]];




InterSeg[sg1_, sg2_] := 
    Module[{seg1, seg2, a1, a2, b1, b2, inter = {}, temp = {}}, 
      If[Chop[Det[{sg1[[2]] - sg1[[1]], sg2[[2]] - sg2[[1]]}]] == 0, 
          Do[If[IsInSeg[sg2, sg1[[i]]], AppendTo[inter, sg1[[i]]], 0], {i, 
              2}];
          
          Do[If[IsInSeg[sg1, sg2[[i]]], AppendTo[inter, sg2[[i]]], 0], {i, 
              2}];
          Return[CleanPoly[inter]];, 
          If[! (vertical[sg1] || vertical[sg2]), seg1 = sg1; seg2 = sg2;
            
            a1 = (seg1[[2]][[2]] - seg1[[1]][[2]])/(seg1[[2]][[1]] - 
                    seg1[[1]][[1]]);
            
            a2 = (seg2[[2]][[2]] - seg2[[1]][[2]])/(seg2[[2]][[1]] - 
                    seg2[[1]][[1]]);
            b1 = seg1[[1]][[2]] - a1*seg1[[1]][[1]];
            b2 = seg2[[1]][[2]] - a2*seg2[[1]][[1]];
            inter = {(b2 - b1)/(a1 - a2), a1*(b2 - b1)/(a1 - a2) + b1};
            If[! And @@ Map[NumberQ, inter], Return[{}], 0];, 
            If[! vertical[sg1], seg1 = sg2; seg2 = sg1, seg1 = sg1; 
              seg2 = sg2];
            
            a2 = (seg2[[2]][[2]] - seg2[[1]][[2]])/(seg2[[2]][[1]] - 
                    seg2[[1]][[1]]);
            b2 = seg2[[1]][[2]] - a2*seg2[[1]][[1]];
            inter = {seg1[[1]][[1]], a2*seg1[[1]][[1]] + b2};];
          
          If[IsInSeg[seg1, inter] && IsInSeg[seg2, inter], 
            Return[Chop[inter]], Return[{}]]];];




AugmentPoly[poly1_, poly2_] := 
    Module[{currseg1, currseg2, poly1a = poly1, poly2a = poly2, inter}, 
      Do[currseg1 = {poly1[[i]], poly1[[Mod[i, Length[poly1]] + 1]]};
        currseg2 = {poly2[[j]], poly2[[Mod[j, Length[poly2]] + 1]]};
        inter = InterSeg[currseg1, currseg2];
        If[Length[inter] != 0, 
          If[Depth[inter] == 2, AppendTo[poly1a, inter]; 
              AppendTo[poly2a, inter], 
              Do[AppendTo[poly1a, inter[[k]]]; 
                  AppendTo[poly2a, inter[[k]]], {k, Length[inter]}];];, 
          0];, {i, Length[poly1]}, {j, Length[poly2]}];
      Return[{CleanPoly[SortVertices[Chop[poly1a]]], 
          CleanPoly[SortVertices[Chop[poly2a]]]}];];




PolygonIntersection[poly1_, poly2_] := 
    Module[{vertices, aib = {}}, 
      vertices = Flatten[AugmentPoly[poly1, poly2], 1];
      Do[If[IsInPoly[poly1, vertices[[i]]] && IsInPoly[poly2, vertices[[i]]], 
            AppendTo[aib, vertices[[i]]], 0];, {i, Length[vertices]}];
      Return[CleanPoly[SortVertices[Chop[aib]]]];];


bary[poly_] := 1/Length[poly] Plus @@ poly;

ApplyIso[iso_, xo_] := iso[[1]].xo + iso[[2]];

ComposeIsos[f_, g_] := {f[[1]].g[[1]], f[[1]].g[[2]] + f[[2]]};

invertpwi[pwi_] := Block[{newpolys = {}, newisos = {}}, Do[
        AppendTo[newpolys, 
          SortVertices[Map[ApplyIso[pwi[[i, 2]], #] &, pwi[[i, 1]]]]];
        AppendTo[
          newisos, {Inverse[
              pwi[[i, 2, 1]]], -Inverse[pwi[[i, 2, 1]]].pwi[[i, 2, 2]]}];
        , {i, Length[pwi]}];
      Return[Transpose[Chop[{newpolys, newisos}]]];
      ];


ApplyPid[pid_, xo_] := 
    Module[{image = {}}, 
      Do[If[(IsInPoly[pid[[i]][[1]], xo]) && Length[image] == 0, 
          image = ApplyIso[pid[[i]][[2]], xo] // Chop; Break[];, 0], {i, 
          Length[pid]}];
      If[Length[image] != 0, Return[image], 
        Print["Point in no atom ! ", N[xo, 20]]];];

ApplyPidToPoly[pid_, poly_] := 
    Module[{image = {}}, 
      Do[If[(IsInPoly[pid[[i]][[1]], bary[poly]]) && Length[image] == 0, 
          image = Chop[Map[ApplyIso[pid[[i]][[2]], #] &, poly]]; Break[];, 
          0], {i, Length[pid]}];
      If[Length[image] != 0, Return[image], 
        Print["center in no atom ! ", N[xo, 20]]];];

Atom[pid_, xo_] := 
    Do[If[IsInPoly[pid[[i, 1]], xo], Return[i], 0], {i, Length[pid]}];

IterPoly[pid_, poly_] := 
    Module[{center = bary[poly], newpoly = {}, reslist = {}, inter = {}}, 
      Do[If[IsInPoly[pid[[i]][[1]], center], 
            newpoly = 
              SortVertices[Chop[Map[ApplyIso[pid[[i]][[2]], #] &, poly]]]; 
            Break[];, 0];, {i, Length[pid]}];
      Do[inter = PolygonIntersection[newpoly, pid[[i]][[1]]];
        If[Length[inter] > 2, AppendTo[reslist, inter], 0];, {i, 
          Length[pid]}];
      Return[reslist];];

IterPolyn[pid_, poly_, n_] := 
    Module[{tofollow = {poly}, compl = {}}, 
      Do[tofollow = Flatten[Map[IterPoly[pid, #] &, tofollow], 1];
        AppendTo[compl, Length[tofollow]];, {i, n}];
      Return[{tofollow, compl}];];

Complexity[pid_, n_] := 
    Plus @@ Map[IterPolyn[pid, #, n][[2]] &, Transpose[pid][[1]]];

Partitionshade[pid_] := 
    Graphics[Table[{Hue[i/Length[pid], 0.2, 1], Polygon[pid[[i, 1]]]}, {i, 
          Length[pid]}]];

DrawPartition[pid_, n_, as_] := 
    Module[{fp, prt}, 
      fp = Flatten[
          Table[IterPolyn[pid, pid[[i, 1]], n][[1]], {i, Length[pid]}], 1];
      Show[{Graphics[
            Table[{Hue[i/Length[pid], 0.2, 1], Polygon[pid[[i, 1]]]}, {i, 
                Length[pid]}]], Map[GraphPoly, fp]}, AspectRatio -> as, 
        PlotRange -> All];];

DrawPartitionQuiet[pid_, n_] := 
    Module[{fp, prt}, 
      fp = Flatten[
          Table[IterPolyn[pid, pid[[i, 1]], n][[1]], {i, Length[pid]}], 1];
      Return[{Graphics[
            Table[{Hue[i/Length[pid], 0.2, 1], Polygon[pid[[i, 1]]]}, {i, 
                Length[pid]}]], Map[GraphPoly, fp]}]];

FirstReturn[pid_, ipid_, poly_, limit_] := 
    Module[{tofollow = {poly}, newfollow = {}, itpol = {}, res = {}, 
        count = 0, backtrack = {}, currpoly = {}, backres = {}}, 
      While[(Length[tofollow] > 0) && (count < limit), newfollow = {};
        Do[itpol = IterPoly[pid, tofollow[[i]]]; count++;
          
          Do[If[! IsInPoly[poly, bary[itpol[[j]]]], 
                AppendTo[newfollow, itpol[[j]]], 
                AppendTo[res, itpol[[j]]]];, {j, Length[itpol]}];, {i, 
            Length[tofollow]}];
        tofollow = newfollow;];
      If[(count >= limit) && (Length[tofollow] != 0), 
        Print["Warning : Recursion Limit Reached, questionable result !"], 
        0];
      Print["Max return time :", count];
      backtrack = Chop[Map[ApplyPidToPoly[ipid, #] &, res]];
      Do[currpoly = backtrack[[i]];
        While[! IsInPoly[poly, bary[currpoly]], 
          currpoly = ApplyPidToPoly[ipid, currpoly];];
        AppendTo[backres, currpoly];, {i, Length[backtrack]}];
      Return[{backres, res}];];

PWI[pid_, ipid_, begin_, end_] := 
    Module[{apilist = {}, curriso, currpoly, iapilist = {}, icurriso, 
        icurrpoly, curratom, currcode = {}, codelst = {}}, 
      Do[currpoly = begin[[i]]; curriso = {{{1, 0}, {0, 1}}, {0, 0}}; 
        notyet = True;
        currcode = {};
        While[notyet, curratom = Atom[pid, bary[currpoly]];
          AppendTo[currcode, curratom];
          curriso = ComposeIsos[pid[[curratom, 2]], curriso];
          currpoly = Chop[Map[ApplyIso[pid[[curratom, 2]], #] &, currpoly]];
          
          notyet = ! 
              Or @@ Table[
                  IsInPoly[end[[j]], bary[currpoly]], {j, Length[end]}];];
        AppendTo[apilist, curriso];
        AppendTo[codelst, currcode];, {i, Length[begin]}];
      Do[icurrpoly = end[[i]]; icurriso = {{{1, 0}, {0, 1}}, {0, 0}}; 
        notyet = True;
        While[notyet, curratom = Atom[ipid, bary[icurrpoly]];
          icurriso = ComposeIsos[ipid[[curratom, 2]], icurriso];
          
          icurrpoly = 
            Chop[Map[ApplyIso[ipid[[curratom, 2]], #] &, icurrpoly]];
          
          notyet = ! 
              Or @@ Table[
                  IsInPoly[begin[[j]], bary[icurrpoly]], {j, 
                    Length[begin]}];];
        AppendTo[iapilist, icurriso];, {i, Length[end]}];
      Return[{Transpose[{begin, apilist}], Transpose[{end, iapilist}], 
          codelst}];];

PWIAnim[pid_, ipid_, begin_, end_] := 
    Module[{apilist = {}, curriso, currpoly, iapilist = {}, icurriso, 
        icurrpoly, curratom, foreground, background, xmin, xmax, ymin, ymax, 
        as}, foreground = DrawPartitionQuiet[pid, 15];
      background = Partitionshade[pid];
      xmin = Min[Transpose[Flatten[Transpose[pid][[1]], 1]][[1]]];
      xmax = Max[Transpose[Flatten[Transpose[pid][[1]], 1]][[1]]];
      ymin = Min[Transpose[Flatten[Transpose[pid][[1]], 1]][[2]]];
      ymax = Max[Transpose[Flatten[Transpose[pid][[1]], 1]][[2]]];
      as = (ymax - ymin)/(xmax - xmin);  
      Do[currpoly = begin[[i]]; curriso = # &; notyet = True;
        While[notyet, 
          Show[Flatten[{background, DrawFilledPolysQuiet[begin, 1], 
                Graphics[{Hue[i/Length[begin]], Polygon[currpoly]}], 
                foreground}, 1], AspectRatio -> as, 
            PlotRange -> {{xmin, xmax}, {ymin, ymax}}, 
            DisplayFunction -> $DisplayFunction];
          curratom = Atom[pid, bary[currpoly]];
          curriso = pid[[curratom, 2]] /@ curriso;
          currpoly = Chop[Map[pid[[curratom, 2]], currpoly]];
          
          notyet = ! 
              Or @@ Table[
                  IsInPoly[end[[j]], bary[currpoly]], {j, Length[end]}];];
        AppendTo[apilist, curriso];
        Show[{background, DrawFilledPolysQuiet[begin, 1], 
            Graphics[{Hue[i/Length[begin]], Polygon[currpoly]}], foreground}, 
          AspectRatio -> as, PlotRange -> {{xmin, xmax}, {ymin, ymax}}, 
          DisplayFunction -> $DisplayFunction];, {i, Length[begin]}];];


PWIAnim2[pid_, begin_, end_] := 
    Module[{currpolys = begin, returned = Table[False, {i, Length[begin]}], 
        curratom, background, foreground, xmin, xmax, ymin, ymax, as}, 
      foreground = DrawPartitionQuiet[pid, 15];
      background = Partitionshade[pid];
      xmin = Min[Transpose[Flatten[Transpose[pid][[1]], 1]][[1]]];
      xmax = Max[Transpose[Flatten[Transpose[pid][[1]], 1]][[1]]];
      ymin = Min[Transpose[Flatten[Transpose[pid][[1]], 1]][[2]]];
      ymax = Max[Transpose[Flatten[Transpose[pid][[1]], 1]][[2]]];
      as = (ymax - ymin)/(xmax - xmin);  
      Show[{background, DrawFilledPolysQuiet[currpolys], foreground}, 
        AspectRatio -> as, PlotRange -> {{xmin, xmax}, {ymin, ymax}}, 
        DisplayFunction -> $DisplayFunction];  
      While[! And @@ returned, 
        Do[If[! returned[[i]], 
            currpolys[[i]] = ApplyPidToPoly[pid, currpolys[[i]]], 0];
          
          If[currpolys[[i]] == end[[i]], returned[[i]] = True, 
            returned[[i]] = False];, {i, Length[begin]}];
        Show[{background, DrawFilledPolysQuiet[currpolys], foreground}, 
          AspectRatio -> as, PlotRange -> {{xmin, xmax}, {ymin, ymax}}, 
          DisplayFunction -> $DisplayFunction];];];

Null
